//+------------------------------------------------------------------+
//|                                          PolyFitR-Squared_v1.mq4 |
//|                                Copyright  2008, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers   1
#property indicator_color1    LightBlue
#property indicator_width1    2
#property indicator_level1    10 
#property indicator_level3    20
#property indicator_level4    80
#property indicator_maximum   100
#property indicator_minimum   0

//---- external variables
extern int     Price       =   0;   //Apply to Price(0-Close;1-Open;2-High;3-Low;4-Median;5-Typical;6-Weighted)
extern int     Degree      =   1;   //Degree of a Polynomial(no more 12)
extern int     MA_Length   =   1;   //Period of Preliminary Smoothing
extern int     MA_Mode     =   0;   //Mode of MA:0-SMA,1-EMA,2-Wilders(SMMA),3-LWMA
extern int     CountBars   = 500;   //Maximum Number of Bars(0-all Bars) 
extern double  CycleMultiplier =   1; 
extern double  CyclePeriodAlpha = 0.07;
//---- indicator buffers
double R2[];
//---- global variables
double PriceArray[];
int cBars;
//+---------------------------------------------------------------------------+
//| Custom indicator initialization function                                  |
//+---------------------------------------------------------------------------+
int init()
{
   IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS )+2);
    
   SetIndexStyle(0, DRAW_LINE);
   SetIndexBuffer(0, R2);
   
   if(CountBars == 0) cBars = Bars;
   else cBars = CountBars;
   
   SetIndexDrawBegin( 0,Bars-cBars+2*(CycleMultiplier+MA_Length));
   
   string short_name="PolyFitR-Squared cycle period adapted("+Price+","+CycleMultiplier+","+Degree+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"R-Squared");
   
   ArrayResize(PriceArray, CycleMultiplier);
   
   return(0);
}

double PolyFit(double& price[],int deg,int len,int bar)
{
double   result=0,Sum=0;
double   AX[12,12],BX[12],CX[12],ZX[12],Pow[24];
int      Row[12];
  
// FILL MATRIX FIRST
   if (len<=1)	Sum=price[0];
   else
   {	
   
   for (int j=0;j<=deg+1;j++) BX[j]=0;
		
		for (int k=1;k<=len;k++) 
  	   {
  	   double YK=price[len-k];
  	   double XK=k;
  	   double Prod=1;
  	      for (j=1;j<=deg+1;j++)
    	   {
    	   BX[j]+= YK*Prod;
    	   Prod*=XK;
  		   }
      }
      
   for (j=1;j<=2*deg;j++) Pow[j]=0;
	
	Pow[0]=len;
	
      for (k=1;k<=len;k++)
	   {
  	   XK=k;
  	   Prod=k;
  	      for (j=1;j<=2*deg;j++)
  	      {
    	   Pow[j]+=Prod;
    	   Prod*=XK;
  		   }
      }	
	
	   for (j=1;j<=deg+1;j++)
	   {	
         for (int l=1;l<=deg+1;l++) 
  	      AX[j,l]=Pow[j+l-2];
      }	
//NOW SOLVE FOR COEFFICIENTS

	for (j=1;j<=deg+1;j++) Row[j]=j;
  	
      for (int i=1;i<=deg;i++)
  	   {
  	      for (k=i+1;k<=deg+1;k++)
         {
            if(MathAbs(AX[Row[k],i]) > MathAbs(AX[Row[i],i]))
      	   {	
      	   int Temp=Row[i];
      	   Row[i]=Row[k];
      	   Row[k]=Temp;
            }
         }

         for (k=i+1;k<=deg+1;k++)
    	   {
    	   if(AX[Row[i],i]!=0) AX[Row[k],i]=AX[Row[k],i]/AX[Row[i],i];
    		   for (l=i+1;l<=deg+1;l++)
      	   AX[Row[k],l]=AX[Row[k],l]-AX[Row[k],i]*AX[Row[i],l];
    		}
		}

   ZX[1]=BX[Row[1]];
      
      for (k=2;k<=deg+1;k++)
  	   {
  	   Sum=0;
      for (l=1;l<=k-1;l++) Sum+=AX[Row[k],l]*ZX[l];
  	   
  	   ZX[k]=BX[Row[k]]-Sum;
	   }
     
	CX[deg+1]=ZX[deg+1]/AX[Row[deg+1],deg+1];
      	
      for (k=deg;k>=1;k--)
  	   {
  	   Sum=0;
  	   l=k+1;  		
  	   for (l=k+1;l<=deg+1;l++) Sum += AX[Row[k],l]*CX[l];
  	   CX[k]=(ZX[k]-Sum)/AX[Row[k],k];
	   }
//NOW COMPUTE NEXT POINT IN SERIES AND RETURN}
   
   Sum=CX[deg+1];
	for (k=deg;k>=1;k--) {Sum=CX[k]+Sum*(len+bar);}
   }
   return (Sum);
}	
//+------------------------------------------------------------------+
//| PolyFitR-Squared_v1                                                |
//+------------------------------------------------------------------+
int start()
{
   int i, j, shift, counted_bars=IndicatorCounted(), limit;
        
   
   if ( counted_bars < 0 )  return(0);
   if ( counted_bars ==0 )  limit=cBars-1; 
         
   limit = Bars - counted_bars; 
   int len = CycleMultiplier+MA_Length;
   
   if ( counted_bars < 1)
   { 
      for(i=1;i<len;i++)
      R2[cBars-i]=EMPTY_VALUE;
   }   
           
   for(shift=limit;shift>=0;shift-- )
   {
      if(shift < cBars)
      {
        double Sum=0;
        double period = MathMax(MathRound(CycleMultiplier*iCustom(NULL,0,"CyclePeriod_v2",Price,CyclePeriodAlpha,0,shift)),3);
        Comment(period);
         ArrayResize(PriceArray, period);
        for(j=0;j<period;j++)
         { 
         PriceArray[j] = iMA(NULL,0,MA_Length,0,MA_Mode,Price,shift + j);
         Sum += PriceArray[j];
         }
         double Yavg = Sum/period; 
         
         double Sum1=0;
         double Sum2=0;
         for(j=period-1;j>=0;j--)
         {
         double poly = PolyFit(PriceArray,Degree,period,-j); 
         double del1 = PriceArray[j] - poly; 
         double del2 = PriceArray[j] - Yavg;
         Sum1 += del1*del1;
         Sum2 += del2*del2; 
         }
         if(Sum2!=0) R2[shift] = 100*(1-(Sum1/Sum2));            
      }
   }

   return(0);
}
//+---------------------------------------------------------------------------+


